;/*******************************************************************************
;*                                                                              *
;*   Abstract    : This file contains interrupt vector and startup code.        *
;*                                                                              *
;*   Functions   : Reset_Handler                                                *
;*                                                                              *
;*   Target      : Ambiq Apollo microcontrollers                                *
;*                                                                              *
;*   Environment : IAR Embedded Workbench                                       *
;*                                                                              *
;*   Distribution: The file is distributed "as is," without any warranty        *
;*                 of any kind.                                                 *
;*                                                                              *
;*******************************************************************************/


                MODULE  ?cstartup

                ;; Forward declaration of sections.
                SECTION CSTACK:DATA:NOROOT(3)

                SECTION .intvec:CODE:NOROOT(2)

                EXTERN  __iar_program_start
                EXTERN  SystemInit
                PUBLIC  __vector_table

                DATA
__vector_table  DCD     sfe(CSTACK)               ; Top of Stack
		        DCD     Reset_Handler             ; Reset
                DCD     NMI_Handler               ; NMI
                DCD     HardFault_Handler         ; Hard Fault
                DCD     MemManage_Handler         ; MPU Fault
                DCD     BusFault_Handler          ; Bus Fault
                DCD     UsageFault_Handler        ; Usage Fault
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall
                DCD     DebugMon_Handler          ; Debug Monitor
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV
                DCD     SysTick_Handler           ; SysTick

; Numbered IRQ handler vectors

; Note: renaming to device dependent ISR function names

                DCD     BROWNOUT_IRQHandler
                DCD     WDT_IRQHandler
                DCD     CLKGEN_IRQHandler
                DCD     VCOMP_IRQHandler
                DCD     IOSLAVE_IRQHandler
                DCD     IOSLAVEACC_IRQHandler
                DCD     IOMSTR0_IRQHandler
                DCD     IOMSTR1_IRQHandler
                DCD     ADC_IRQHandler
                DCD     GPIO_IRQHandler
                DCD     CTIMER_IRQHandler
                DCD     UART_IRQHandler
                


                THUMB
; Dummy Exception Handlers (infinite loops which can be modified)

                PUBWEAK Reset_Handler
                SECTION .text:CODE:REORDER:NOROOT(2)
Reset_Handler

                LDR.W R0, =0xE000ED88            ; CPACR is located at address 0xE000ED88
                LDR R1, [R0]                     ; Read CPACR
                ORR R1, R1, #(0xF << 20)         ; Set bits 20-23 to enable CP10 and CP11 coprocessors
                STR R1, [R0]                     ; Write back the modified value to the CPACR

                LDR     R0, =SystemInit
                BLX     R0
                LDR     R0, =__iar_program_start
                BX      R0
                
                PUBWEAK NMI_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
NMI_Handler
                B       NMI_Handler

                PUBWEAK HardFault_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
HardFault_Handler
                B       HardFault_Handler

                PUBWEAK MemManage_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
MemManage_Handler
                B       MemManage_Handler

                PUBWEAK BusFault_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
BusFault_Handler
                B       BusFault_Handler

                PUBWEAK UsageFault_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
UsageFault_Handler
                B       UsageFault_Handler

                PUBWEAK SVC_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
SVC_Handler
                B       SVC_Handler

                PUBWEAK DebugMon_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
DebugMon_Handler
                B       DebugMon_Handler

                PUBWEAK PendSV_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
PendSV_Handler
                B       PendSV_Handler

                PUBWEAK SysTick_Handler
                SECTION .text:CODE:REORDER:NOROOT(1)
SysTick_Handler
                B       SysTick_Handler

                
                PUBWEAK BROWNOUT_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
BROWNOUT_IRQHandler
                B       BROWNOUT_IRQHandler

                PUBWEAK WDT_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
WDT_IRQHandler
                B       WDT_IRQHandler

                PUBWEAK CLKGEN_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
CLKGEN_IRQHandler
                B       CLKGEN_IRQHandler

                PUBWEAK VCOMP_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
VCOMP_IRQHandler
                B       VCOMP_IRQHandler

                PUBWEAK IOSLAVE_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
IOSLAVE_IRQHandler
                B       IOSLAVE_IRQHandler

                PUBWEAK IOSLAVEACC_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
IOSLAVEACC_IRQHandler
                B       IOSLAVEACC_IRQHandler

                PUBWEAK IOMSTR0_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
IOMSTR0_IRQHandler
                B       IOMSTR0_IRQHandler

                PUBWEAK IOMSTR1_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
IOMSTR1_IRQHandler
                B       IOMSTR1_IRQHandler

                PUBWEAK ADC_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
ADC_IRQHandler
                B       ADC_IRQHandler

                PUBWEAK GPIO_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
GPIO_IRQHandler
                B       GPIO_IRQHandler

                PUBWEAK CTIMER_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
CTIMER_IRQHandler
                B       CTIMER_IRQHandler

                PUBWEAK UART_IRQHandler
                SECTION .text:CODE:REORDER:NOROOT(1)
UART_IRQHandler
                B       UART_IRQHandler


                PUBWEAK Dummy
                SECTION .text:CODE:REORDER:NOROOT(1)
Dummy
                B       Dummy

                END
